@@ -1,8 +1,8 @@
 # coding: utf-8
 
-VERSION = (0, 10, 21)
-CODENAME = "zip it"
-BUILD_DT = (2021, 5, 16)
+VERSION = (0, 11, 0)
+CODENAME = "the grid"
+BUILD_DT = (2021, 5, 29)
 
 S_VERSION = ".".join(map(str, VERSION))
 S_BUILD_DT = "{0:04d}-{1:02d}-{2:02d}".format(*BUILD_DT)
